﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Text.RegularExpressions;

namespace ExpressionsSHP
{
    public partial class MainForm : Form
    {
        private const String layerName = "expressionsSHP_Lyr";

        public MainForm()
        {
            InitializeComponent();
        }

        private void FillFieldNames()
        {
            XrMapLib.ShapeMapLayer lyr = xR.Layers.GetLayerAsShapeMap(layerName);
            XrMapLib.FieldSet fieldSet = lyr.AttributeTable.FieldSet;
            int cntFields = fieldSet.FieldCount;

            cbFieldNames.Items.Clear();
            for (int iField = 0; iField < cntFields; ++iField)
            {
                String fieldName = fieldSet.GetFieldName(iField);
                cbFieldNames.Items.Add(fieldName);
            }
        }

        private void btnOpenSHP_Click(object sender, EventArgs e)
        {
            DialogResult DR = openFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                xR.StopMapDrawing();
                xR.WaitForAllDrawing();

                xR.Layers.RemoveAllLayer();

                String fileName = openFileDialog.FileName;
                xR.Layers.AddShapeMapLayer(layerName, fileName);

                xR.WaitForAllConnections();
                xR.ZoomFullExtent();
                xR.Update();

                FillFieldNames();
            }
        }

        private void btnOutputFileName_Click(object sender, EventArgs e)
        {
            DialogResult DR = saveFileDialog.ShowDialog();
            if (DR == DialogResult.OK)
            {
                String fileName = saveFileDialog.FileName;
                tbOutputFileName.Text = fileName;
            }
        }

        private XrMapLib.FieldSetDefine GetFieldDefine(XrMapLib.IAttributeTable table)
        {
            XrMapLib.FieldSet fieldSet = table.FieldSet;

            XrMapLib.FieldSetDefine FSD = new XrMapLib.FieldSetDefine();
            int cntFields = fieldSet.FieldCount;
            for (int iField = 0; iField < cntFields; ++iField)
            {
                XrMapLib.XrFieldTypeEnum type = fieldSet.GetFieldType(iField);
                String fieldName = fieldSet.GetFieldName(iField);
                if (type == XrMapLib.XrFieldTypeEnum.XrDouble || type == XrMapLib.XrFieldTypeEnum.XrFloat)
                {
                    FSD.AddFloatField(fieldName, 10, 2);
                }
                else if (type == XrMapLib.XrFieldTypeEnum.XrInteger || type == XrMapLib.XrFieldTypeEnum.XrShort || type == XrMapLib.XrFieldTypeEnum.XrVeryShort)
                {
                    FSD.AddIntegerField(fieldName, 10);
                }
                else if (type == XrMapLib.XrFieldTypeEnum.XrString)
                {
                    FSD.AddStringField(fieldName, 25);
                }
            }

            return FSD;
        }

        private XrMapLib.ValueList GetAttributeValueList(XrMapLib.IAttributeRow row, int fieldsCount)
        {
            XrMapLib.ValueList values = new XrMapLib.ValueList();

            for (int iField = 0; iField < fieldsCount; ++iField)
            {
                String value = row.GetValueAsString(iField);
                values.AddValueByString(value);
            }

            return values;
        }

        private XrMapLib.CoordinateListCollection GetShapeCoordList(XrMapLib.IShapeRow row)
        {
            XrMapLib.CoordinateListCollection CLC = new XrMapLib.CoordinateListCollection();

            int cntParts = row.PartCount;
            for (int iPart = 0; iPart < cntParts; ++iPart)
            {
                int cntVertices = row.GetVertexCountFromPart(iPart);
                XrMapLib.CoordinateList CL = new XrMapLib.CoordinateList();  
                for (int iVertex = 0; iVertex < cntVertices; ++iVertex)
                {
                    XrMapLib.Coordinate coord = row.GetVertexFromPart(iPart, iVertex);
                    CL.Add(coord.X, coord.Y);
                }

                CLC.Add(CL);
            }

            return CLC;
        }

        private XrMapLib.Coordinate GetShapeCoord(XrMapLib.IShapeRow row)
        {
            XrMapLib.Coordinate coord = row.GetVertex(0);
            return coord;
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            if (tbOutputFileName.Text.Length == 0 ||
                cbFieldNames.SelectedIndex == -1 ||
                cbExpression.Text.Length == 0 ||
                xR.Layers.Count == 0)
            {
                MessageBox.Show("입력 레이어나 검색 대상 필드명, 정규표현식 또는 저장할 SHP 파일명을 지정해 주시기 바랍니다.");
                return;
            }

            xR.StopMapDrawing();
            xR.WaitForAllDrawing();

            XrMapLib.IShapeMapLayer lyr = xR.Layers.GetLayerAsShapeMap(layerName);
            XrMapLib.IShapeTable shpTbl = lyr.ShapeTable;
            XrMapLib.IAttributeTable attTbl = lyr.AttributeTable;
            XrMapLib.FieldSetDefine FSD = GetFieldDefine(attTbl);
            XrMapLib.Tool tool = new XrMapLib.Tool();
            int fieldsCount = attTbl.FieldSet.FieldCount;
            XrMapLib.ValueList VL;
            XrMapLib.CoordinateListCollection CLC;
            int cntRows = shpTbl.RowCount;
            XrMapLib.IShapeRow shp = null;
            Regex reg = new Regex(cbExpression.Text);
            XrMapLib.XrShapeTypeEnum ShapeType = lyr.ShapeType;
            XrMapLib.Coordinate Coord;

            String OutputFileName = tbOutputFileName.Text;
            XrMapLib.EsriShpFileHandler SFH = tool.CreateEsriShpFileHandler(OutputFileName, lyr.ShapeType, FSD);
            if (SFH == null)
            {
                MessageBox.Show("Error: Creating shp file");
                return;
            }

            tools.Enabled = false;
            progressBar.Visible = true;
            int fieldIndex = cbFieldNames.SelectedIndex;

            try
            {
                for (int FID = 0; FID < cntRows; ++FID)
                {
                    shp = shpTbl.GetRow(FID);

                    XrMapLib.IAttributeRow attr = attTbl.GetRow(FID);
                    if (attr.Load())
                    {
                        String value = attr.GetValueAsString(fieldIndex);

                        if (reg.IsMatch(value))
                        {
                            VL = GetAttributeValueList(attr, fieldsCount);

                            if (shp.Load())
                            {
                                if (ShapeType == XrMapLib.XrShapeTypeEnum.XrPoint)
                                {
                                    Coord = GetShapeCoord(shp);
                                    SFH.AddPoint(Coord.X, Coord.Y, VL);
                                }
                                else
                                {
                                    CLC = GetShapeCoordList(shp);
                                    SFH.AddMultiPart(CLC, VL);
                                }


                                shp.Unload();
                            }
                        }

                        attr.Unload();
                    }
                }
            } 
            catch(Exception exception)
            {
                MessageBox.Show(exception.Message);
                return;
            }

            SFH.Close();

            xR.Layers.RemoveAllLayer();
            xR.Layers.AddShapeMapLayer(layerName, OutputFileName);

            xR.WaitForAllConnections();
            xR.ZoomFullExtent();

            progressBar.Visible = false;
            tools.Enabled = true;
        }
    }
}
